//Serial Communication Class Heaer File 

#ifndef _SerialComBase_H_
#define _SerialComBase_H_

//Standard Input/Output Definitions for C++
#include <iostream>
//Unix Standard Function Definitions
#include <unistd.h>
//File Control Definitions
#include <fcntl.h>
//POSIX terminal control definitions
#include <termios.h>

//definition of parity flags
#define NOPARITY 0
#define ODDPARITY 1
#define EVENPARITY 2

//definition of receiving delimiter pattern 
#define CRNL 0
#define NLCR 1
#define NOTHING 2

using namespace std;

class SerialComBase{
 private:
  char *dev;
  int fd;     //file descriptor for dev

  //Structure for saving old setting
  struct termios oldtio;
  //Structure for Saving now setting
  struct termios newtio;
  //Paramitter of BaudRate
  unsigned int BaudRate;
  //Parameter of DataBit
  int DataBit;
  //Parameter of Delimiter
  int Delimiter;
   
 public:
  SerialComBase();
  bool OpenCOMDevice(const char* dev);
  bool CloseCOMDevice();

  ///////////////////////////////////////////////////////////////////////////
  // Communication methods for RS232C communication 
  ///////////////////////////////////////////////////////////////////////////
  //Send Data for Target Device through Specified COM Port
  bool SendMessage(char* data, unsigned int Datalength);
  
  //Receive Data from Specified COM Port
  //Return Value:(int) Receive Data Length
  int ReceiveMessage(char *buf);

  ///////////////////////////////////////////////////////////////////////////
  //Setting for RS232C Communication Parameter
  ///////////////////////////////////////////////////////////////////////////
  //Set Communication Speed
  void SetBaudRate(unsigned int speed);
  //Get Communication Speed
  unsigned int GetBaudRate();

  //Set Data Bit(5,6,7or8bit)
  void SetDataBit(int bit);
  int  GetDataBit();
  //Set Stop bit(1 or 2)
  void SetStopBit2();	//Stop bit 2
  void ClearStopBit2();	//Stop bit 1

  //Set parity flag, when you want to check the received data.
  void SetParityFlag(unsigned int flag); //flag are defined above
  //Clear Parity Flag
  void ClearParityFlag();

  //Enable to read serial RX data
  void EnableRead();
  //Disable to read Serial RX data
  void DisableRead();

  //Canonical Mode
  void EnableCanonical(); //1 Line receive mode
  void DisableCanonical();//

  //Local Connect Mode
  void EnableLocalConnect();
  void DisableLocalConnect();

  //Delimiter Mode
  void SetReceiveDelimiter(int mode);
  int  GetReceiveDelimiter();
};

#endif
